import 'dart:async';
import 'dart:convert';

import 'package:at_client/at_client.dart';
import 'package:at_client/src/manager/monitor.dart';
import 'package:at_client/src/preference/monitor_preference.dart';
import 'package:at_client/src/response/notification_response_parser.dart';
import 'package:at_client/src/transformer/response_transformer/notification_response_transformer.dart';
import 'package:test/test.dart';

import 'test_util.dart';

void main() {
  var currentAtSign = '@sitaram';
  var namespace = 'wavi';
  var sharedWithAtSign = '@murali';

  /// The purpose of this test is to verify the notify text with setting
  /// shouldEncrypt parameter to true (which encrypt the notify text)
  /// and setting shouldEncrypt to false (text message is sent as plain text).
  group('A group of tests to verify notification text', () {
    var notifyText = 'Hello This is ';
    var inputToExpectedOutput = {
      // Encrypt the notify text data
      NotificationParams.forText(
              '$notifyText${currentAtSign.replaceAll('@', '')}',
              sharedWithAtSign,
              shouldEncrypt: true):
          '$sharedWithAtSign:$notifyText${currentAtSign.replaceAll('@', '')}',
      // Send notify text message as plain text
      NotificationParams.forText(
              '$notifyText${currentAtSign.replaceAll('@', '')}',
              sharedWithAtSign,
              shouldEncrypt: false):
          '$sharedWithAtSign:$notifyText${currentAtSign.replaceAll('@', '')}'
    };
    inputToExpectedOutput.forEach((input, expectedOutput) {
      test('Setting shouldEncrypt to ${input.atKey.metadata?.isEncrypted}',
          () async {
        // Setting the AtClientManager instance to current atsign
        await AtClientManager.getInstance().setCurrentAtSign(
            currentAtSign, namespace, TestUtil.getSitaramPreference());

        var epochMillsNow = DateTime.now().millisecondsSinceEpoch;
        var notificationResult = await AtClientManager.getInstance()
            .notificationService
            .notify(input);
        expect(notificationResult.notificationStatusEnum,
            NotificationStatusEnum.delivered);

        // Setting the AtClientManager instance to sharedWith atsign
        await AtClientManager.getInstance().setCurrentAtSign(
            sharedWithAtSign, namespace, TestUtil.getMuraliPreference());
        // Getting the notification via the monitor command
        StreamController<AtNotification> controller = StreamController();
        bool isNotificationReceived = false;
        var monitorForNotification = MonitorForNotification(
            notificationResult.notificationID,
            sharedWithAtSign,
            epochMillsNow,
            controller);
        await monitorForNotification.init();
        controller.stream.listen((data) async {
          isNotificationReceived = true;
          var response =
              await NotificationResponseTransformer().transform(Tuple()
                ..one = data
                ..two = (NotificationConfig()..shouldDecrypt = true));
          expect(response.key, expectedOutput);
        });
        while (!isNotificationReceived) {
          await Future.delayed(Duration(milliseconds: 100));
        }
      });
    });
  });
}

/// Class responsible for getting the notifications from the cloud secondary
class MonitorForNotification {
  String notificationId;
  String atSign;
  int epochMillsNow;
  StreamController streamController;
  Monitor? monitor;

  MonitorForNotification(this.notificationId, this.atSign, this.epochMillsNow,
      this.streamController);

  init() async {
    monitor = Monitor(
        _onMonitorSuccess,
        _onMonitorError,
        atSign,
        TestUtil.getMuraliPreference(),
        MonitorPreference()
          ..lastNotificationTime = epochMillsNow
          ..keepAlive = false,
        _onMonitorRetry);

    await monitor?.start(lastNotificationTime: epochMillsNow);
  }

  Future<void> _onMonitorSuccess(String notificationStr) async {
    if (notificationStr.contains(notificationId)) {
      final notificationResponseParser = NotificationResponseParser();
      final atNotifications =
          await notificationResponseParser.getAtNotifications(
              notificationResponseParser.parse(notificationStr));
      for (var element in atNotifications) {
        streamController.add(element);
      }
      monitor?.stop();
    }
  }

  //Dummy implementation for error
  _onMonitorError(arg1) {
    print(arg1);
  }

  // Dummy implementation for retry callback
  _onMonitorRetry() {}
}
